@@ -60,9 +60,9 @@ class _OpenStackCloudMixin:
     :param bool strict: Only return documented attributes for each resource
                         as per the Data Model contract. (Default False)
     """
-    _OBJECT_MD5_KEY = 'x-object-meta-x-sdk-md5'
-    _OBJECT_SHA256_KEY = 'x-object-meta-x-sdk-sha256'
-    _OBJECT_AUTOCREATE_KEY = 'x-object-meta-x-sdk-autocreated'
+    _OBJECT_MD5_KEY = 'x-sdk-md5'
+    _OBJECT_SHA256_KEY = 'x-sdk-sha256'
+    _OBJECT_AUTOCREATE_KEY = 'x-sdk-autocreated'
     _OBJECT_AUTOCREATE_CONTAINER = 'images'
 
     # NOTE(shade) shade keys were x-object-meta-x-shade-md5 - we need to check
@@ -102,26 +102,15 @@ class _OpenStackCloudMixin:
         cache_class = self.config.get_cache_class()
         cache_arguments = self.config.get_cache_arguments()
 
-        self._resource_caches = {}
+        self._cache_expirations = dict()
 
         if cache_class != 'dogpile.cache.null':
             self.cache_enabled = True
-            self._cache = self._make_cache(
-                cache_class, cache_expiration_time, cache_arguments)
-            expirations = self.config.get_cache_expirations()
-            for expire_key in expirations.keys():
-                # Only build caches for things we have list operations for
-                if getattr(
-                        self, 'list_{0}'.format(expire_key), None):
-                    self._resource_caches[expire_key] = self._make_cache(
-                        cache_class, expirations[expire_key], cache_arguments)
-
-            self._SERVER_AGE = DEFAULT_SERVER_AGE
-            self._PORT_AGE = DEFAULT_PORT_AGE
-            self._FLOAT_AGE = DEFAULT_FLOAT_AGE
         else:
             self.cache_enabled = False
 
+            # TODO(gtema): delete it with the standalone cloud layer caching
+
             def _fake_invalidate(unused):
                 pass
 
@@ -148,15 +137,20 @@ class _OpenStackCloudMixin:
                     new_func.invalidate = _fake_invalidate
                     setattr(self, method, new_func)
 
-        # If server expiration time is set explicitly, use that. Otherwise
-        # fall back to whatever it was before
-        self._SERVER_AGE = self.config.get_cache_resource_expiration(
-            'server', self._SERVER_AGE)
-        self._PORT_AGE = self.config.get_cache_resource_expiration(
-            'port', self._PORT_AGE)
-        self._FLOAT_AGE = self.config.get_cache_resource_expiration(
-            'floating_ip', self._FLOAT_AGE)
+        # Uncoditionally create cache even with a "null" backend
+        self._cache = self._make_cache(
+            cache_class, cache_expiration_time, cache_arguments)
+        expirations = self.config.get_cache_expirations()
+        for expire_key in expirations.keys():
+            self._cache_expirations[expire_key] = \
+                expirations[expire_key]
 
+        # TODO(gtema): delete in next change
+        self._SERVER_AGE = 0
+        self._PORT_AGE = 0
+        self._FLOAT_AGE = 0
+
+        self._api_cache_keys = set()
         self._container_cache = dict()
         self._file_hash_cache = dict()
 
@@ -331,7 +325,8 @@ class _OpenStackCloudMixin:
             name_key = '%s:%s' % (self.name, namespace)
 
         def generate_key(*args, **kwargs):
-            arg_key = ','.join(args)
+            # TODO(frickler): make handling arg keys actually work
+            arg_key = ''
             kw_keys = sorted(kwargs.keys())
             kwargs_key = ','.join(
                 ['%s:%s' % (k, kwargs[k]) for k in kw_keys if k != 'cache'])
@@ -712,9 +707,11 @@ class _OpenStackCloudMixin:
     def get_name(self):
         return self.name
 
-    def get_session_endpoint(self, service_key):
+    def get_session_endpoint(self, service_key, **kwargs):
+        if not kwargs:
+            kwargs = {}
         try:
-            return self.config.get_session_endpoint(service_key)
+            return self.config.get_session_endpoint(service_key, **kwargs)
         except keystoneauth1.exceptions.catalog.EndpointNotFound as e:
             self.log.debug(
                 "Endpoint not found in %s cloud: %s", self.name, str(e))
@@ -731,7 +728,7 @@ class _OpenStackCloudMixin:
                     error=str(e)))
         return endpoint
 
-    def has_service(self, service_key):
+    def has_service(self, service_key, version=None):
         if not self.config.has_service(service_key):
             # TODO(mordred) add a stamp here so that we only report this once
             if not (service_key in self._disable_warnings
@@ -742,7 +739,12 @@ class _OpenStackCloudMixin:
                 self._disable_warnings[service_key] = True
             return False
         try:
-            endpoint = self.get_session_endpoint(service_key)
+            kwargs = dict()
+            # If a specific version was requested - try it
+            if version is not None:
+                kwargs['min_version'] = version
+                kwargs['max_version'] = version
+            endpoint = self.get_session_endpoint(service_key, **kwargs)
         except exc.OpenStackCloudException:
             return False
         if endpoint:
